classdef measure < handle
    % Generate measurement and input vector 
    %   Use acceleration measurements from IMU and position and orientation 
    %   measurements from Qualisys
    
    properties
        std_acc = 0.01; 
        std_range = 0.1; 
        l 
        y_prev
        a_prev 
        a
        y
        att
        R
        pos

        T = [1 0 0;
            0.0230 1 0;
            -0.0834 0.0173 1];

        S = 1e-5*[0.1005 0 0;
            0 0.0813 0;
            0 0 -0.0494];

        b = [-0.0408;
            0.0026;
            -0.1295]; 


    end
    
    methods
        
        function get_measurements(mes, qualisys, imu, i, trans)

        % Obtain orientation 
        
        psi   = qualisys(5,i);
        theta = qualisys(6,i);
        phi   = qualisys(7,i); 
        
        mes.att = [phi; theta; psi]; 
    
        if ~isnan(psi) && ~isnan(theta) && ~isnan(phi) 
            
            % Define rotation matrix 
            mes.R = Rzyx(phi, theta, psi);
            
            mes.a = mes.T*mes.S*([imu(2,i); imu(3,i) ;imu(4,i)]- mes.b); 
           
            mes.a_prev = mes.a; 

        else
    
            mes.a = mes.a_prev; 
            mes.R = eye(3);
            
        end
    
        % Range measurements (position in case of linear filter)
        
        mes.pos = [qualisys(2,i); qualisys(3,i); qualisys(4,i)] - mes.R*mes.l; 
        
        xq = mes.pos(1,1);
        yq = mes.pos(2,1);
        zq = mes.pos(3,1); 

        % Check if measurement drop-out occured for Qualisys

        if ~isnan(xq) && ~isnan(yq) && ~isnan(zq) 

            noise = zeros(4,1); 
            
            mes.y(1) = norm(mes.pos - trans.p1) + noise(1);
            mes.y(2) = norm(mes.pos - trans.p2) + noise(2);
            mes.y(3) = norm(mes.pos - trans.p3) + noise(3);
            mes.y(4) = norm(mes.pos - trans.p4) + noise(4);
            
            mes.y_prev = mes.y; 
            
        else
            
            mes.y = mes.y_prev;
            mes.R = eye(3); 
        
        end
        
        end
        
    end
    
end

