classdef range < handle
 
    properties
        xm 
        xm_0
        x2
        zm
        M = [1 0 0 0;
             0 1 0 0;
             0 0 1 0;
             0 0 0 0];
         
        l = ones(4,1); 
        b = 1; % true bias value  
        y 
        R
        Cy
        r1
        z
        noise_0 = zeros(5,1);
        std_range = 0.000104207; 
        
    end
    
    methods
        
        function update(r, sim, trans)
            
            r.y(1) = norm(sim.posn - trans.p1);
            r.y(2) = norm(sim.posn - trans.p2);
            r.y(3) = norm(sim.posn - trans.p3);
            r.y(4) = norm(sim.posn - trans.p4); 
                   
        end
        
        function update2(r,trans,mes) % range measurements with noise 
             

            Cyp = [2*trans.p1'; 2*trans.p2'; 2*trans.p3'; 2*trans.p4'];
            

            ro1 = mes.y(1); 
            ro2 = mes.y(2); 
            ro3 = mes.y(3); 
            ro4 = mes.y(4); 
            
            
            y12 = mes.y(1)^2- r.std_range^2; 
            y22 = mes.y(2)^2- r.std_range^2; 
            y32 = mes.y(3)^2- r.std_range^2;
            y42 = mes.y(4)^2- r.std_range^2; 
            
            Cyb = [y12; y22; y32; y42]; 
            r.Cy = [Cyp Cyb]; 
            
            r.z = [norm(trans.p1)^2, norm(trans.p2)^2, norm(trans.p3)^2, norm(trans.p4)^2]'; 
            c = inv(r.Cy)*r.l; 
            w = inv(r.Cy)*r.z; 
            h = 2*c'*r.M*w-1;
            r.r1 = ((-h + sqrt(h^2-4*c'*r.M*c*w'*r.M*w))/(2*c'*r.M*c)); 
            r2 = ((-h - sqrt(h^2-4*c'*r.M*c*w'*r.M*w))/(2*c'*r.M*c)); 
            
            % Resolve ambiguity 
            r.xm = real(r.r1*c + w);
            r.x2 = real(r2*c + w); 
            
            error1 = 1 - abs(r.xm(4));
            error2 = 1 - abs(r.x2(4)); 
            
            if abs(error1) > abs(error2)
                r.x2 = r.xm; 
                r.xm = real(r2*c + w);
                r.r1 = r2; 
            end

            r.R(1,1) = (2/(r.b^2))*(2*ro1^2*r.std_range^2 + r.std_range^4); 
            r.R(2,2) = (2/(r.b^2))*(2*ro2^2*r.std_range^2 + r.std_range^4); 
            r.R(3,3) = (2/(r.b^2))*(2*ro3^2*r.std_range^2 + r.std_range^4); 
            r.R(4,4) = (2/(r.b^2))*(2*ro4^2*r.std_range^2 + r.std_range^4); 
            
                       
        end
        
        function algtrans(r, sim, trans)
            
            Cyp = [2*trans.p1'; 2*trans.p2'; 2*trans.p3'; 2*trans.p4'];
            
            y12 = (1/r.b)*(sim.posn'*sim.posn - 2*trans.p1'*sim.posn + norm(trans.p1)^2); 
            y22 = (1/r.b)*(sim.posn'*sim.posn - 2*trans.p2'*sim.posn + norm(trans.p2)^2); 
            y32 = (1/r.b)*(sim.posn'*sim.posn - 2*trans.p3'*sim.posn + norm(trans.p3)^2); 
            y42 = (1/r.b)*(sim.posn'*sim.posn - 2*trans.p4'*sim.posn + norm(trans.p4)^2); 
            
            Cyb = [y12; y22; y32; y42]; 
            r.Cy = [Cyp Cyb]; 
            
            r.z = [norm(trans.p1)^2, norm(trans.p2)^2, norm(trans.p3)^2, norm(trans.p4)^2]'; %obs! might not be properly transposed for matrix??! 
            c = inv(r.Cy)*r.l; 
            w = inv(r.Cy)*r.z; 
            h = 2*c'*r.M*w-1;
            r.r1 = (-h + sqrt(h^2-4*c'*r.M*c*w'*r.M*w))/(2*c'*r.M*c); 
            r2 =  (-h - sqrt(h^2-4*c'*r.M*c*w'*r.M*w))/(2*c'*r.M*c); 
            
            % Resolve ambiguity 
            r.xm = r.r1*c + w;
            
            if abs(r.xm(4)-1) > 0.011 % can change this tolerance later, now it should be correct, as no bias is included
                r.xm = r2*c + w;
                r.r1 = r2; 
            end
            
            
        end
        
        
    end
    
end

