classdef ekf < handle
       
    % EKF
        %  Standard
    
    properties
        
        h 
        
        A
        
        B 
        
        E

        Q
        
        R
  
        P_bar = diag([1e-9 1e-10 1e-10 1e-20 1e-4 1e-4 1e-7 1e-25 1e-25 1e-25]);
        
        h1
        
        H = zeros(4,10); 
        
        K = zeros(10,4); 
        
        x_bar
        
        x_est

        P_est
        
        acu
        
        gn = [0;0;0]; %9.81]; 
        
        std_range = 1.6376e-04; %0.00010389; %0.000104207; 
        
        var_acc_x = 1.6765331232158308e-04;%e-04;
        
        var_acc_y = 1.623284546288902e-04;%e-04; 
        
        var_acc_z = 3.342784513438293e-04; %e-04; 
        
        std_bias_x = 1e-4; %1e-10; 
        
        std_bias_y = 1e-4; 
        
        std_bias_z = 1e-4;
        
    end
    
    methods

        function update(est2,trans, mes) 
            
            % Define LTV system 
            
            est2.A = [zeros(3,4) eye(3) zeros(3,3);
                      zeros(1,10);
                      zeros(3,7) -mes.R;
                      zeros(3,10)];  
 
            est2.E = [zeros(3,7); 
                      1 zeros(1,6); 
                      zeros(3,1) -mes.R zeros(3,3); 
                      zeros(3,4) eye(3)];
        
            est2.B = [zeros(4,3) zeros(4,3); 
                      mes.R eye(3); 
                      zeros(3,3) zeros(3,3)];
        
            % Find discrete matrices for A, E and B
            
            [phi, gamma] = c2d(est2.A, est2.E, est2.h); 
            [~, delta] = c2d(est2.A, est2.B, est2.h);
            
            % Find sensor noise covariance matrix, R

            est2.R = est2.std_range^2*eye(4); 
            
            % Find process noise covariance matrix, Q
            
            est2.Q = diag([1e-6 est2.var_acc_x est2.var_acc_y est2.var_acc_z est2.std_bias_x^2 est2.std_bias_y^2 est2.std_bias_z^2]);
            
           if est2.acu == 50 
                
               if mes.dropout == 0
                   
                rho1 = norm(est2.x_bar(1:3) - trans.p1);
                rho2 = norm(est2.x_bar(1:3) - trans.p2);
                rho3 = norm(est2.x_bar(1:3) - trans.p3);
                rho4 = norm(est2.x_bar(1:3) - trans.p4);

                est2.H(1,:) = [(est2.x_bar(1)-trans.p1(1))/(sqrt(est2.x_bar(4))*rho1),(est2.x_bar(2)-trans.p1(2))/(sqrt(est2.x_bar(4))*rho1), (est2.x_bar(3)-trans.p1(3))/(sqrt(est2.x_bar(4))*rho1),-0.5*rho1*(1/((est2.x_bar(4))^(3/2))),0,0,0,0,0,0];
                est2.H(2,:) = [(est2.x_bar(1)-trans.p2(1))/(sqrt(est2.x_bar(4))*rho2),(est2.x_bar(2)-trans.p2(2))/(sqrt(est2.x_bar(4))*rho2), (est2.x_bar(3)-trans.p2(3))/(sqrt(est2.x_bar(4))*rho2),-0.5*rho2*(1/((est2.x_bar(4))^(3/2))),0,0,0,0,0,0];
                est2.H(3,:) = [(est2.x_bar(1)-trans.p3(1))/(sqrt(est2.x_bar(4))*rho3),(est2.x_bar(2)-trans.p3(2))/(sqrt(est2.x_bar(4))*rho3), (est2.x_bar(3)-trans.p3(3))/(sqrt(est2.x_bar(4))*rho3),-0.5*rho3*(1/((est2.x_bar(4))^(3/2))),0,0,0,0,0,0];
                est2.H(4,:) = [(est2.x_bar(1)-trans.p4(1))/(sqrt(est2.x_bar(4))*rho4),(est2.x_bar(2)-trans.p4(2))/(sqrt(est2.x_bar(4))*rho4), (est2.x_bar(3)-trans.p4(3))/(sqrt(est2.x_bar(4))*rho4),-0.5*rho4*(1/((est2.x_bar(4))^(3/2))),0,0,0,0,0,0];

                % Compute Kalman gain factor

                est2.K = est2.P_bar*est2.H'*inv(((est2.H*est2.P_bar*est2.H')+ est2.R));

                est2.h1(1) = (1/sqrt(est2.x_bar(4)))*rho1;
                est2.h1(2) = (1/sqrt(est2.x_bar(4)))*rho2;
                est2.h1(3) = (1/sqrt(est2.x_bar(4)))*rho3;
                est2.h1(4) = (1/sqrt(est2.x_bar(4)))*rho4;
            
                est2.x_est = est2.x_bar + est2.K*(mes.y - est2.h1'); 
                est2.P_est = (eye(10)-(est2.K*est2.H))*est2.P_bar*(eye(10)-(est2.K*est2.H))' + est2.K*est2.R*est2.K';

                est2.acu = 0; 
                
               else
                   
                est2.K = zeros(10,4);
                est2.P_est = est2.P_bar; 
                est2.x_est = est2.x_bar; 
               
                est2.acu = 0;
                 
               end
               
           else
                
                est2.K = zeros(10,4);
                est2.P_est = est2.P_bar; 
                est2.x_est = est2.x_bar; 
                
                est2.acu = est2.acu + 1; 
               
            end            
              
            % Prediction for state vector and covariance:
            
            u = [mes.a; est2.gn]; 
            est2.x_bar = phi*est2.x_est + delta*u;  
            est2.P_bar = phi*est2.P_est*phi' + gamma*est2.Q*gamma'; 
            
        end        
        
    end
    
end

