classdef simulator < handle
    % Simulator used to define motion of vehicle
        %   Generates acceleration measurements used in filters
        
    properties 
        posn
        q
        h
        velb
        acc_n
        std_acc = 0.01; 
        veln
    end
    
    methods
        function identify(sim, new)
             sim.h = sim.h + new; 
             disp(['This is the new time ', num2str(sim.h)]); 
        end
        
        function update(sim, input)
            
            [~, Rnb, T] = quatern(sim.q); % Computes the quaternion linear and angular velocity transformation matrix
            sim.veln = Rnb*(sim.velb + input.vel);   % Finding translational velocities in NED from velocities in BODY 
            %sim.acc_n = Smtrx(input.w)*Rnb*sim.velb+Rnb*input.dvel; %Can be used as acceleration measurements. Add noise by adding std_acc*randn(1)
            sim.acc_n = Smtrx(input.w)*Rnb*sim.velb+Rnb*input.dvel + randn(3,1)*sim.std_acc;%sim.noise;
        
            % Integrate in order to get values of the trajectory at the next point
            % in time using Euler integration 
            
            sim.posn = sim.posn + sim.h*sim.veln; 
            sim.velb = sim.velb + sim.h*input.dvel;
            sim.q    = sim.q + sim.h*T*input.w;

            % Unit quaternion normalization 

            sim.q = sim.q/norm(sim.q); 

        end

        
        
    end
    
end

