classdef lkf < handle
    % Linearized Kalman filter 
        %   Stage 3 of TSF
    
    properties
        
        h 
        
        A
        
        B
        
        E
      
        Q
        
        R

        P_bar = diag([0.1^3 0.1^3 0.1^3 1e-1 1e-4 1e-4 1e-4 1e-5 1e-5 1e-5]);

        h1
        
        H = zeros(4,10); 
        
        K = zeros(10,4); 
        
        x_bar
        
        x_est

        P_est
        
        acu
        
        gn = [0;0;9.81]; 

        std_range = 0.0001; %0.000104207; 
        
        var_acc_x = 1e-4; %1e-2; %7*1e-3; %1.6765331232158308e-04;%e-04; % 1e-3
        
        var_acc_y = 1e-4;%1e-2; %7*1e-3; %1.623284546288902e-04;%e-04; % 1e-3
        
        var_acc_z = 1e-3; %5*1e-1; %1e-2; %3.342784513438293e-04; %e-04; % 1e-3
        
        std_bias_x = 1e-4;%1e-6; %1e-3;%0.01; %1e-10; 
        
        std_bias_y = 1e-4;%1e-5; %1e-3; %0.01; 
        
        std_bias_z = 1e-4; %1e-6; %1e-3; %0.01; 
        
    end
    
    methods

        function update(est,trans,ltv,input)
            
             % Define LTV system 
        
             est.A = [zeros(3,4) eye(3) zeros(3,3);
                      zeros(1,10);
                      zeros(3,7) -input.rot;
                      zeros(3,10)];  
 
             est.E = [zeros(3,7); 
                      1 zeros(1,6); 
                      zeros(3,1) -input.rot zeros(3,3); 
                      zeros(3,4) eye(3)];
                  
             est.B = [zeros(4,3) zeros(4,3); 
                      input.rot eye(3); 
                      zeros(3,3) zeros(3,3)];
                  
            % Find discrete matrices for A, E and B
            
            [phi, gamma] = c2d(est.A, est.E, est.h); 
            [~, delta] = c2d(est.A, est.B, est.h); 
            
            % Find sensor noise covariance matrix, R

            est.R = est.std_range^2*eye(4);
            
             % Find process noise covariance matrix, Q
             
            est.Q = diag([1e-7 est.var_acc_x est.var_acc_y est.var_acc_z est.std_bias_x^2 est.std_bias_y^2 est.std_bias_z^2]);
    
            if input.range 
                
                rho1 = norm(ltv.x_est(1:3) - trans.p1);
                rho2 = norm(ltv.x_est(1:3) - trans.p2);
                rho3 = norm(ltv.x_est(1:3) - trans.p3);
                rho4 = norm(ltv.x_est(1:3) - trans.p4);

                est.H(1,:) = [(ltv.x_est(1)-trans.p1(1))/(sqrt(ltv.x_est(4))*rho1),(ltv.x_est(2)-trans.p1(2))/(sqrt(ltv.x_est(4))*rho1), (ltv.x_est(3)-trans.p1(3))/(sqrt(ltv.x_est(4))*rho1),-0.5*rho1*(1/((ltv.x_est(4))^(3/2))),0,0,0,0,0,0];
                est.H(2,:) = [(ltv.x_est(1)-trans.p2(1))/(sqrt(ltv.x_est(4))*rho2),(ltv.x_est(2)-trans.p2(2))/(sqrt(ltv.x_est(4))*rho2), (ltv.x_est(3)-trans.p2(3))/(sqrt(ltv.x_est(4))*rho2),-0.5*rho2*(1/((ltv.x_est(4))^(3/2))),0,0,0,0,0,0];
                est.H(3,:) = [(ltv.x_est(1)-trans.p3(1))/(sqrt(ltv.x_est(4))*rho3),(ltv.x_est(2)-trans.p3(2))/(sqrt(ltv.x_est(4))*rho3), (ltv.x_est(3)-trans.p3(3))/(sqrt(ltv.x_est(4))*rho3),-0.5*rho3*(1/((ltv.x_est(4))^(3/2))),0,0,0,0,0,0];
                est.H(4,:) = [(ltv.x_est(1)-trans.p4(1))/(sqrt(ltv.x_est(4))*rho4),(ltv.x_est(2)-trans.p4(2))/(sqrt(ltv.x_est(4))*rho4), (ltv.x_est(3)-trans.p4(3))/(sqrt(ltv.x_est(4))*rho4),-0.5*rho4*(1/((ltv.x_est(4))^(3/2))),0,0,0,0,0,0];

                est.h1 = (1/sqrt(ltv.x_est(4)))*[rho1; rho2; rho3; rho4] + est.H*(est.x_bar - ltv.x_est);

                % Compute Kalman gain factor
                est.K = est.P_bar*est.H'*inv(((est.H*est.P_bar*est.H')+ est.R));
            
                est.x_est = est.x_bar + est.K*(input.y'- est.h1); 
                est.P_est = (eye(10)-(est.K*est.H))*est.P_bar*(eye(10)-(est.K*est.H))' + est.K*est.R*est.K';
                
                input.range = 0; 
 
            else
                
                est.K = zeros(10,4);
                est.P_est = est.P_bar; 
                est.x_est = est.x_bar; 
                
            end            
              
            % Prediction for state vector and covariance:
            
            u = [input.acc'; est.gn]; 
            est.x_bar = phi*est.x_est + delta*u;  
            est.P_bar = phi*est.P_est*phi' + gamma*est.Q*gamma'; 
            
        end        
        
    end
    
end

