classdef measure < handle
    % Generate measurement and input vector 
    %   Use acceleration measurements from IMU and position and orientation 
    %   measurements from Qualisys
    
    properties
        std_acc = 0.01; 
        std_range = 0.1; 
        l = [ -0.09; -0.007; 0]; 
        y_prev
        a_prev 
        a
        y
        att
        R
    end
    
    methods
        
        function get_measurements(mes, qualisys, imu, i, trans)

        % Obtain orientation 
        
        psi   = qualisys(5,i);
        theta = qualisys(6,i);
        phi   = qualisys(7,i); 
        
        mes.att = [phi; theta; psi]; 
    
        if ~isnan(psi) && ~isnan(theta) && ~isnan(phi) 
            
            % Define rotation matrix 
            mes.R = Rzyx(phi, theta, psi);
            
            mes.a = [imu(2,i) imu(3,i) imu(4,i)]'; 
            mes.a = mes.a -mes.l;
            mes.a_prev = mes.a; 
            
        else
  
            mes.a = mes.a_prev; 
            mes.R = eye(3);
            
        end
    
        % Range measurements (position in case of linear filter)
      
        xq = qualisys(2,i);
        yq = qualisys(3,i);
        zq = -qualisys(4,i); 

        % Check if measurement drop-out occured for Qualisys

        if ~isnan(xq) && ~isnan(yq) && ~isnan(zq) 
            
            pos = [xq yq zq]'; 
            
             
            noise = zeros(4,1); 
            
            mes.y(1) = norm(pos - trans.p1) + noise(1);
            mes.y(2) = norm(pos - trans.p2) + noise(2);
            mes.y(3) = norm(pos - trans.p3) + noise(3);
            mes.y(4) = norm(pos - trans.p4) + noise(4);
            
            mes.y_prev = mes.y; 
            
        else
            
            mes.y = mes.y_prev;
            mes.R = eye(3); 
            
        end
        
        end
        
    end
    
end

