classdef ltvk < handle 
    % LTV Kalman filter 
    %   Stage 2 of TSF
    
    properties
        h 
        
        A
        
        B
        
        E
      
        Q
        
        P_bar = diag([5*1e-4 5*1e-4 5*1e-5 1e-5 1e-4 1e-4 1e-4 1e-4 1e-4 1e-4]);

        h1
        
        K = zeros(10,4); 
        
        x_bar
        
        x_est

        P_est
        
        acu
        
        gn = [0;0;9.81]; 
        
        con
        
    end
    
    methods
        
         function update(ltv,r,sim, trans) 
             
            % Define LTV system, input and noise matrices
            
            ltv.A = [zeros(3,3) zeros(3,1) eye(3) zeros(3,3);
                     zeros(1,10);
                     zeros(3,7) -sim.Rnb;
                     zeros(3,10)];  
 
            ltv.E = [zeros(3,1) zeros(3,3) zeros(3,3);
                     1 zeros(1,3) zeros(1,3);
                     zeros(3,1) -sim.Rnb zeros(3,3);
                     zeros(3,1) zeros(3,3) eye(3)];
            
            ltv.B = [zeros(4,3) zeros(4,3); 
                      sim.Rnb eye(3); 
                      zeros(3,3) zeros(3,3)];
        
             % Find discrete matrices for A, E and B
            
            [phi, gamma] = c2d(ltv.A, ltv.E, ltv.h); 
            [~, delta] = c2d(ltv.A, ltv.B, ltv.h);
            
            % Define time-varying measurement matrix 
            
            C = [r.Cy zeros(4,6)];
            
            ltv.con = cond(C);
           
             % Find process noise covariance matrix, Q
        
            ltv.Q = diag([1e-4 sim.std_acc^2 sim.std_acc^2 sim.std_acc^2 sim.std_bias^2 sim.std_bias^2 sim.std_bias^2]); 
            
            if ltv.acu == 50
                
                % Compute Kalman gain factor

                ltv.K = ltv.P_bar*C'*inv(C*ltv.P_bar*C' + r.R); 

                % Constructed measurement

                Z = r.z;
                D = -r.l; 
                
                Z_bar = C*ltv.x_bar + D*r.r1; 

                % Estimate update

                ltv.x_est = ltv.x_bar + ltv.K*(Z - Z_bar);
                ltv.P_est = (eye(10) - ltv.K*C)*ltv.P_bar*(eye(10)-ltv.K*C)' + ltv.K*r.R*ltv.K';   
                
                ltv.acu = 0; 
                
            else 
                
                ltv.K = zeros(10,4); 
                ltv.x_est = ltv.x_bar;
                ltv.P_est = ltv.P_bar; 
                
                ltv.acu = ltv.acu + 1; 
                
            end
            
            % Estimate propagation 
            
            u = [sim.acc_b; ltv.gn]; 
            ltv.x_bar = phi*ltv.x_est + delta*u; 
            ltv.P_bar = phi*ltv.P_est*phi' + gamma*ltv.Q*gamma'; 
         
         end
         
    end
    
end

